@@ -36,12 +36,14 @@ import optparse
 import subprocess
 import shutil
 import re
+import tarfile
 
 
 # CVS and Subversion command line client commands
 CVS_CMD = 'cvs'
 SVN_CMD = 'svn'
 HG_CMD = 'hg'
+GIT_CMD = 'git'
 
 
 def pipe(cmd):
@@ -51,12 +53,24 @@ def pipe(cmd):
   status = child.wait()
   return (output, status)
 
+
 def cmd_failed(cmd, output, status):
   print 'CMD FAILED:', ' '.join(cmd)
   print 'Output:'
   sys.stdout.write(output)
   raise RuntimeError('%s command failed!' % cmd[0])
 
+
+def split_output(self, cmd):
+  (output, status) = pipe(cmd)
+  if status:
+    cmd_failed(cmd, output, status)
+  retval = output.split(os.linesep)[:-1]
+  if retval and not retval[-1]:
+    del retval[-1]
+  return retval
+
+
 class CvsRepos:
   def __init__(self, path):
     """Open the CVS repository at PATH."""
@@ -86,14 +100,14 @@ class CvsRepos:
     the HEAD revision, or any valid CVS revision string to export that
     revision."""
     os.mkdir(dest_path)
-    cmd = [ CVS_CMD, '-Q', '-d', ':local:' + self.cvsroot, 'export' ]
+    cmd = [CVS_CMD, '-Q', '-d', ':local:' + self.cvsroot, 'export']
     if rev:
-      cmd.extend([ '-r', rev ])
+      cmd.extend(['-r', rev])
     else:
-      cmd.extend([ '-D', 'now' ])
+      cmd.extend(['-D', 'now'])
     if keyword_opt:
       cmd.append(keyword_opt)
-    cmd.extend([ '-d', dest_path, self.module ])
+    cmd.extend(['-d', dest_path, self.module])
     (output, status) = pipe(cmd)
     if status or output:
       cmd_failed(cmd, output, status)
@@ -130,7 +144,7 @@ class SvnRepos:
   def export(self, path, dest_path):
     """Export PATH to DEST_PATH."""
     url = '/'.join([self.url, path])
-    cmd = [ SVN_CMD, 'export', '-q', url, dest_path ]
+    cmd = [SVN_CMD, 'export', '-q', url, dest_path]
     (output, status) = pipe(cmd)
     if status or output:
       cmd_failed(cmd, output, status)
@@ -149,14 +163,11 @@ class SvnRepos:
 
   def list(self, path):
     """Return a list of all files and directories in PATH."""
-    cmd = [ SVN_CMD, 'ls', self.url + '/' + path ]
-    (output, status) = pipe(cmd)
-    if status:
-      cmd_failed(cmd, output, status)
+    cmd = [SVN_CMD, 'ls', self.url + '/' + path]
     entries = []
-    for line in output.split("\n"):
+    for line in split_output(cmd):
       if line:
-        entries.append(line[:-1])
+        entries.append(line.rstrip('/'))
     return entries
 
   def tags(self):
@@ -167,6 +178,7 @@ class SvnRepos:
     """Return a list of all branches in the repository."""
     return self.branch_list
 
+
 class HgRepos:
   name = 'hg'
 
@@ -197,10 +209,7 @@ class HgRepos:
     if not os.path.exists(dest_path):
       cmd = self.base_cmd + ['manifest', '--rev', rev]
 
-      (output, status) = pipe(cmd)
-      if status:
-        cmd_failed(cmd, output, status)
-      manifest = [fn for fn in output.split("\n")[:-1]
+      manifest = [fn for fn in split_output(cmd)
                   if not fn.startswith('.hg')]
       if not manifest:
         os.mkdir(dest_path)
@@ -221,14 +230,14 @@ class HgRepos:
 
   def tags(self):
     cmd = self.base_cmd + ['tags', '-q']
-    tags = self._split_output(cmd)
+    tags = split_output(cmd)
     tags.remove('tip')
     return tags
 
   def branches(self):
     if self._branches is None:
       cmd = self.base_cmd + ['branches', '-q']
-      self._branches = branches = self._split_output(cmd)
+      self._branches = branches = split_output(cmd)
       try:
         branches.remove('default')
         self._have_default = True
@@ -237,17 +246,98 @@ class HgRepos:
 
     return self._branches
 
-  def _split_output(self, cmd):
-    (output, status) = pipe(cmd)
-    if status:
-      cmd_failed(cmd, output, status)
-    return output.split("\n")[:-1]
 
 class GitRepos:
   name = 'git'
 
   def __init__(self, path):
-    raise NotImplementedError()
+    self.path = path
+    self.repo_cmd = [
+        GIT_CMD,
+        '--git-dir=' + os.path.join(self.path, '.git'),
+        '--work-tree=' + self.path,
+        ]
+
+    self._branches = None               # cache result of branches()
+    self._have_master = None           # so export_trunk() doesn't blow up
+
+  def __str__(self):
+    return os.path.basename(self.path)
+
+  def _export(self, dest_path, rev):
+    # clone the repository
+    cmd = [GIT_CMD, 'archive', '--remote=' + self.path, '--format=tar', rev]
+    git_proc = subprocess.Popen(cmd, stdout=subprocess.PIPE)
+
+    if False:
+      # Unfortunately for some git tags the below causes
+      # git_proc.wait() to hang.  The git archive process is in a
+      # <defunct> state and the verify-cvs2svn hangs for good.
+      tar = tarfile.open(mode="r|", fileobj=git_proc.stdout)
+      for tarinfo in tar:
+        tar.extract(tarinfo, dest_path)
+      tar.close()
+    else:
+      os.mkdir(dest_path)
+      tar_proc = subprocess.Popen(
+          ['tar', '-C', dest_path, '-x'],
+          stdin=git_proc.stdout, stdout=subprocess.PIPE,
+          )
+      output = tar_proc.stdout.read()
+      status = tar_proc.wait()
+      if output or status:
+        raise RuntimeError(
+            'Git tar extraction of rev %s from repo %s to %s failed (%s)!'
+            % (rev, self.path, dest_path, output)
+            )
+
+    status = git_proc.wait()
+    if status:
+      raise RuntimeError(
+          'Git extract of rev %s from repo %s to %s failed!'
+          % (rev, self.path, dest_path)
+          )
+
+    if not os.path.exists(dest_path):
+      raise RuntimeError(
+          'Git clone of %s to %s failed!' % (self.path, dest_path)
+          )
+
+  def export_trunk(self, dest_path):
+    self.branches()                     # ensure _have_default is set
+    if self._have_master:
+      self._export(dest_path, 'master')
+    else:
+      # same as CVS does when exporting empty trunk
+      os.mkdir(dest_path)
+
+  def export_tag(self, dest_path, tag):
+    self._export(dest_path, tag)
+
+  def export_branch(self, dest_path, branch):
+    self._export(dest_path, branch)
+
+  def tags(self):
+    cmd = self.repo_cmd + ['tag']
+    tags = split_output(cmd)
+    return tags
+
+  def branches(self):
+    if self._branches is None:
+      cmd = self.repo_cmd + ['branch']
+      branches = split_output(cmd)
+      # Remove the two chracters at the start of the branch name
+      for i in range(len(branches)):
+        branches[i] = branches[i][2:]
+      self._branches = branches
+      try:
+        branches.remove('master')
+        self._have_master = True
+      except ValueError:
+        self._have_master = False
+
+    return self._branches
+
 
 def transform_symbol(ctx, name):
   """Transform the symbol NAME using the renaming rules specified
@@ -266,6 +356,12 @@ class Failures(object):
   def __init__(self):
     self.count = 0                      # number of failures seen
 
+  def __str__(self):
+    return str(self.count)
+
+  def __repr__(self):
+    return "<%s at 0x%x: %s>" % (self.__class__.__name__, id(self), self.count)
+
   def report(self, summary, details=None):
     self.count += 1
     sys.stdout.write(' FAIL: %s\n' % summary)
@@ -276,11 +372,14 @@ class Failures(object):
   def __nonzero__(self):
     return self.count > 0
 
+
 def file_compare(failures, base1, base2, run_diff, rel_path):
-  """Compare the mode and contents of two files.  The paths are
-  specified as two base paths BASE1 and BASE2, and a path REL_PATH that
-  is relative to the two base paths.  Return 1 if the file mode and
-  contents are identical, else 0."""
+  """Compare the mode and contents of two files.
+
+  The paths are specified as two base paths BASE1 and BASE2, and a
+  path REL_PATH that is relative to the two base paths.  Return True
+  iff the file mode and contents are identical."""
+
   ok = True
   path1 = os.path.join(base1, rel_path)
   path2 = os.path.join(base2, rel_path)
@@ -301,12 +400,13 @@ def file_compare(failures, base1, base2, run_diff, rel_path):
       if run_diff:
         cmd = ['diff', '-u', path1, path2]
         (output, status) = pipe(cmd)
-        diff = output.split('\n')
+        diff = output.split(os.linesep)
       else:
         diff = None
       failures.report('File contents differ for %s' % rel_path,
                       details=diff)
       ok = False
+      break
     if len(data1) == 0:
       # eof
       break
@@ -315,10 +415,12 @@ def file_compare(failures, base1, base2, run_diff, rel_path):
 
 
 def tree_compare(failures, base1, base2, run_diff, rel_path=''):
-  """Compare the contents of two directory trees, including the contents
-  of all files.  The paths are specified as two base paths BASE1 and BASE2,
-  and a path REL_PATH that is relative to the two base paths.  Return 1
-  if the trees are identical, else 0."""
+  """Compare the contents of two directory trees, including file contents.
+
+  The paths are specified as two base paths BASE1 and BASE2, and a
+  path REL_PATH that is relative to the two base paths.  Return True
+  iff the trees are identical."""
+
   if not rel_path:
     path1 = base1
     path2 = base2
@@ -327,45 +429,52 @@ def tree_compare(failures, base1, base2, run_diff, rel_path=''):
     path2 = os.path.join(base2, rel_path)
   if not os.path.exists(path1):
     failures.report('%s does not exist' % path1)
-    return 0
+    return False
   if not os.path.exists(path2):
     failures.report('%s does not exist' % path2)
-    return 0
+    return False
   if os.path.isfile(path1) and os.path.isfile(path2):
     return file_compare(failures, base1, base2, run_diff, rel_path)
   if not (os.path.isdir(path1) and os.path.isdir(path2)):
     failures.report('Path types differ for %r' % rel_path)
-    return 0
+    return False
   entries1 = os.listdir(path1)
   entries1.sort()
   entries2 = os.listdir(path2)
   entries2.sort()
+
+  ok = True
+
   missing = filter(lambda x: x not in entries2, entries1)
   extra = filter(lambda x: x not in entries1, entries2)
   if missing:
     failures.report('Directory /%s is missing entries: %s' %
                     (rel_path, ', '.join(missing)))
+    ok = False
   if extra:
     failures.report('Directory /%s has extra entries: %s' %
                     (rel_path, ', '.join(extra)))
-  ok = 1
+    ok = False
+
   for entry in entries1:
     new_rel_path = os.path.join(rel_path, entry)
     if not tree_compare(failures, base1, base2, run_diff, new_rel_path):
-      ok = 0
+      ok = False
   return ok
 
 
 def verify_contents_single(failures, cvsrepos, verifyrepos, kind, label, ctx):
-  """Verify that the contents of the HEAD revision of all directories
-  and files in the conversion repository VERIFYREPOS matches the ones in
-  the CVS repository CVSREPOS.  KIND can be either 'trunk', 'tag' or
+  """Verify the HEAD revision of a trunk, tag, or branch.
+
+  Verify that the contents of the HEAD revision of all directories and
+  files in the conversion repository VERIFYREPOS match the ones in the
+  CVS repository CVSREPOS.  KIND can be either 'trunk', 'tag' or
   'branch'.  If KIND is either 'tag' or 'branch', LABEL is used to
   specify the name of the tag or branch.  CTX has the attributes:
   CTX.tmpdir: specifying the directory for all temporary files.
   CTX.skip_cleanup: if true, the temporary files are not deleted.
-  CTX.run_diff: if true, run diff on differing files.
-  """
+  CTX.run_diff: if true, run diff on differing files."""
+
   itemname = kind + (kind != 'trunk' and '-' + label or '')
   cvs_export_dir = os.path.join(
     ctx.tmpdir, 'cvs-export-%s' % itemname)
@@ -389,14 +498,14 @@ def verify_contents_single(failures, cvsrepos, verifyrepos, kind, label, ctx):
     if not tree_compare(
           failures, cvs_export_dir, vrf_export_dir, ctx.run_diff
           ):
-      return 0
+      return False
   finally:
     if not ctx.skip_cleanup:
       if os.path.exists(cvs_export_dir):
         shutil.rmtree(cvs_export_dir)
       if os.path.exists(vrf_export_dir):
         shutil.rmtree(vrf_export_dir)
-  return 1
+  return True
 
 
 def verify_contents(failures, cvsrepos, verifyrepos, ctx):
@@ -410,6 +519,7 @@ def verify_contents(failures, cvsrepos, verifyrepos, ctx):
 
   # Verify contents of trunk
   print 'Verifying trunk'
+  sys.stdout.flush()
   if not verify_contents_single(
         failures, cvsrepos, verifyrepos, 'trunk', None, ctx
         ):
@@ -418,6 +528,7 @@ def verify_contents(failures, cvsrepos, verifyrepos, ctx):
   # Verify contents of all tags
   for tag in verifyrepos.tags():
     print 'Verifying tag', tag
+    sys.stdout.flush()
     if not verify_contents_single(
           failures, cvsrepos, verifyrepos, 'tag', tag, ctx
           ):
@@ -433,8 +544,10 @@ def verify_contents(failures, cvsrepos, verifyrepos, ctx):
             failures, cvsrepos, verifyrepos, 'branch', branch, ctx
             ):
         locations.append('branch:' + branch)
+    sys.stdout.flush()
 
-  assert bool(failures) == bool(locations)
+  assert bool(failures) == bool(locations), \
+         "failures = %r\nlocations = %r" % (failures, locations)
 
   # Show the results
   if failures:
@@ -444,6 +557,8 @@ def verify_contents(failures, cvsrepos, verifyrepos, ctx):
       sys.stdout.write('  %s\n' % location)
   else:
     sys.stdout.write('PASS: %s == %s\n' % (cvsrepos, verifyrepos))
+  sys.stdout.flush()
+
 
 class OptionContext:
   pass
@@ -475,7 +590,7 @@ def main(argv):
                     help='assume verify-repos is hg')
   parser.add_option('--git',
                     action='store_const', dest='repos_type', const='git',
-                    help='assume verify-repos is git (not implemented!)')
+                    help='assume verify-repos is git')
   parser.add_option('--suppress-keywords',
                     action='store_const', dest='keyword_opt', const='-kk',
                     help='suppress CVS keyword expansion '
@@ -552,6 +667,7 @@ def main(argv):
 
   sys.exit(failures and 1 or 0)
 
+
 if __name__ == '__main__':
   main(sys.argv)
 
