@@ -1,7 +1,7 @@
 # (Be in -*- python -*- mode.)
 #
 # ====================================================================
-# Copyright (c) 2007 CollabNet.  All rights reserved.
+# Copyright (c) 2007-2008 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -208,9 +208,9 @@ class AppleSingleFilter(object):
 class CompoundStream(object):
   """A stream that reads from a series of streams, one after the other."""
 
-  def __init__(self, *streams):
+  def __init__(self, streams, stream_index=0):
     self.streams = list(streams)
-    self.stream_index = 0
+    self.stream_index = stream_index
 
   def read(self, size=-1):
     if size < 0:
@@ -254,12 +254,23 @@ def get_maybe_apple_single_stream(stream):
     string_io = StringIO(e.data_read)
     if e.eof:
       # The original stream already reached EOF, so the part already
-      # read contains the complete file contents:
-      return string_io
+      # read contains the complete file contents.  Nevertheless return
+      # a CompoundStream to make sure that the stream's close() method
+      # is called:
+      return CompoundStream([stream, string_io], stream_index=1)
     else:
       # The stream needs to output the part already read followed by
       # whatever hasn't been read of the original stream:
-      return CompoundStream(string_io, stream)
+      return CompoundStream([string_io, stream])
+
+
+def get_maybe_apple_single(data):
+  """Treat DATA as AppleSingle if possible; otherwise treat it literally.
+
+  If DATA is in AppleSingle format, then return its data fork.
+  Otherwise, return the original DATA."""
+
+  return get_maybe_apple_single_stream(StringIO(data)).read()
 
 
 if __name__ == '__main__':
@@ -278,11 +289,10 @@ if __name__ == '__main__':
   #CHUNK_SIZE = -1
   CHUNK_SIZE = 100
 
-  f = get_maybe_apple_single_stream(sys.stdin)
-
   if CHUNK_SIZE < 0:
-    sys.stdout.write(f.read())
+    sys.stdout.write(get_maybe_apple_single(sys.stdin.read()))
   else:
+    f = get_maybe_apple_single_stream(sys.stdin)
     while True:
       s = f.read(CHUNK_SIZE)
       if not s:
