@@ -1,7 +1,7 @@
 # (Be in -*- python -*- mode.)
 #
 # ====================================================================
-# Copyright (c) 2000-2007 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2009 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -17,11 +17,10 @@
 """This module contains common facilities used by cvs2svn."""
 
 
-import re
 import time
 import codecs
 
-from cvs2svn_lib.log import Log
+from cvs2svn_lib.log import logger
 
 
 # Always use these constants for opening databases.
@@ -87,6 +86,16 @@ class CommandError(FatalError):
           % (self.command, self.exit_status))
 
 
+def canonicalize_eol(text, eol):
+  """Replace any end-of-line sequences in TEXT with the string EOL."""
+
+  text = text.replace('\r\n', '\n')
+  text = text.replace('\r', '\n')
+  if eol != '\n':
+    text = text.replace('\n', eol)
+  return text
+
+
 def path_join(*components):
   """Join two or more pathname COMPONENTS, inserting '/' as needed.
   Empty component are skipped."""
@@ -115,54 +124,6 @@ class IllegalSVNPathError(FatalException):
   pass
 
 
-# Control characters (characters not allowed in Subversion filenames):
-ctrl_characters_regexp = re.compile('[\\\x00-\\\x1f\\\x7f]')
-
-
-def verify_svn_filename_legal(filename):
-  """Verify that FILENAME is a legal filename.
-
-  FILENAME is a path component of a CVS path.  Check that it won't
-  choke SVN:
-
-  - Check that it is not empty.
-
-  - Check that it is not equal to '.' or '..'.
-
-  - Check that the filename does not include any control characters.
-
-  If any of these tests fail, raise an IllegalSVNPathError."""
-
-  if filename == '':
-    raise IllegalSVNPathError("Empty filename component.")
-
-  if filename in ['.', '..']:
-    raise IllegalSVNPathError("Illegal filename component %r." % (filename,))
-
-  m = ctrl_characters_regexp.search(filename)
-  if m:
-    raise IllegalSVNPathError(
-        "Character %r in filename %r is not supported by Subversion."
-        % (m.group(), filename,)
-        )
-
-
-def verify_svn_path_legal(path):
-  """Verify that PATH is a legitimate SVN path.
-
-  If not, raise an IllegalSVNPathError."""
-
-  if path.startswith('/'):
-    raise IllegalSVNPathError("Path %r must not start with '/'." % (path,))
-  head = path
-  while head != '':
-    (head,tail) = path_split(head)
-    try:
-        verify_svn_filename_legal(tail)
-    except IllegalSVNPathError, e:
-        raise IllegalSVNPathError('Problem with path %r: %s' % (path, e,))
-
-
 def normalize_svn_path(path, allow_empty=False):
   """Normalize an SVN path (e.g., one supplied by a user).
 
@@ -274,6 +235,25 @@ def verify_paths_disjoint(*paths):
     raise PathsNotDisjointException(problems)
 
 
+def is_trunk_revision(rev):
+  """Return True iff REV is a trunk revision.
+
+  REV is a CVS revision number (e.g., '1.6' or '1.6.4.5').  Return
+  True iff the revision is on trunk."""
+
+  return rev.count('.') == 1
+
+
+def is_branch_revision_number(rev):
+  """Return True iff REV is a branch revision number.
+
+  REV is a CVS revision number in canonical form (i.e., with zeros
+  removed).  Return True iff it refers to a whole branch, as opposed
+  to a single revision."""
+
+  return rev.count('.') % 2 == 0
+
+
 def format_date(date):
   """Return an svn-compatible date string for DATE (seconds since epoch).
 
@@ -283,9 +263,26 @@ def format_date(date):
 
 
 class CVSTextDecoder:
-  """Callable that decodes CVS strings into unicode."""
+  """Callable that decodes CVS strings into Unicode.
 
-  def __init__(self, encodings, fallback_encoding=None):
+  Members:
+
+    decoders -- a list [(name, CodecInfo.decode), ...] containing the
+        names and decoders that will be used in 'strict' mode to
+        attempt to decode inputs.
+
+    fallback_decoder -- a tuple (name, CodecInfo.decode) containing
+        the name and decoder that will be used in 'replace' mode if
+        none of the decoders in DECODERS succeeds.  If no
+        fallback_decoder has been specified, this member contains
+        None.
+
+    eol_fix -- a string to which all EOL sequences will be converted,
+        or None if they should be left unchanged.
+
+  """
+
+  def __init__(self, encodings, fallback_encoding=None, eol_fix=None):
     """Create a CVSTextDecoder instance.
 
     ENCODINGS is a list containing the names of encodings that are
@@ -295,23 +292,47 @@ class CVSTextDecoder:
     should be used as a source encoding in lossy 'replace' mode if all
     of ENCODINGS failed.
 
+    EOL_FIX is the string to which all EOL sequences should be
+    converted.  If it is set to None, then EOL sequences are left
+    unchanged.
+
     Raise LookupError if any of the specified encodings is unknown."""
 
-    self.decoders = [
-        (encoding, codecs.lookup(encoding)[1])
-        for encoding in encodings]
+    self.decoders = []
 
-    if fallback_encoding is None:
+    for encoding in encodings:
+      self.add_encoding(encoding)
+
+    self.set_fallback_encoding(fallback_encoding)
+    self.eol_fix = eol_fix
+
+  def add_encoding(self, encoding):
+    """Add an encoding to be tried in 'strict' mode.
+
+    ENCODING is the name of an encoding.  If it is unknown, raise a
+    LookupError."""
+
+    for (name, decoder) in self.decoders:
+      if name == encoding:
+        return
+    else:
+      self.decoders.append( (encoding, codecs.lookup(encoding)[1]) )
+
+  def set_fallback_encoding(self, encoding):
+    """Set the fallback encoding, to be tried in 'replace' mode.
+
+    ENCODING is the name of an encoding.  If it is unknown, raise a
+    LookupError."""
+
+    if encoding is None:
       self.fallback_decoder = None
     else:
-      self.fallback_decoder = (
-          fallback_encoding, codecs.lookup(fallback_encoding)[1]
-          )
+      self.fallback_decoder = (encoding, codecs.lookup(encoding)[1])
 
-  def __call__(self, s):
+  def decode(self, s):
     """Try to decode string S using our configured source encodings.
 
-    Return the string as a unicode string.  If S is already a unicode
+    Return the string as a Unicode string.  If S is already a unicode
     string, do nothing.
 
     Raise UnicodeError if the string cannot be decoded using any of
@@ -323,13 +344,19 @@ class CVSTextDecoder:
       try:
         return decoder(s)[0]
       except ValueError:
-        Log().verbose("Encoding '%s' failed for string %r" % (name, s))
+        logger.verbose("Encoding '%s' failed for string %r" % (name, s))
 
     if self.fallback_decoder is not None:
       (name, decoder) = self.fallback_decoder
       return decoder(s, 'replace')[0]
     else:
-      raise UnicodeError
+      raise UnicodeError()
+
+  def __call__(self, s):
+    s = self.decode(s)
+    if self.eol_fix is not None:
+      s = canonicalize_eol(s, self.eol_fix)
+    return s
 
 
 class Timestamper:
@@ -365,15 +392,15 @@ class Timestamper:
       # changesets with even earlier timestamps depend on this one.
       self.timestamp = self.timestamp + 1.0
       if not change_expected:
-        Log().warn(
+        logger.warn(
             'Timestamp "%s" is in the future; changed to "%s".'
             % (time.asctime(time.gmtime(timestamp)),
                time.asctime(time.gmtime(self.timestamp)),)
             )
     elif timestamp < self.timestamp + 1.0:
       self.timestamp = self.timestamp + 1.0
-      if not change_expected and Log().is_on(Log.VERBOSE):
-        Log().verbose(
+      if not change_expected and logger.is_on(logger.VERBOSE):
+        logger.verbose(
             'Timestamp "%s" adjusted to "%s" to ensure monotonicity.'
             % (time.asctime(time.gmtime(timestamp)),
                time.asctime(time.gmtime(self.timestamp)),)
