@@ -1,7 +1,7 @@
 # (Be in -*- python -*- mode.)
 #
 # ====================================================================
-# Copyright (c) 2000-2007 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2008 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -28,26 +28,25 @@ DEFAULT_TAGS_BASE = 'tags'
 SVNADMIN_EXECUTABLE = 'svnadmin'
 CO_EXECUTABLE = 'co'
 CVS_EXECUTABLE = 'cvs'
-SORT_EXECUTABLE = 'sort'
 
 # A pickled list of the projects defined for this conversion.
 PROJECTS = 'projects.pck'
 
-# A file holding the Serializer to be used for
-# CVS_REVS_SUMMARY_*_DATAFILE and CVS_SYMBOLS_SYMMARY_*_DATAFILE:
-SUMMARY_SERIALIZER = 'summary-serializer.pck'
+# A file holding the Serializer to be used for CVS_REVS_*_DATAFILE and
+# CVS_SYMBOLS_*_DATAFILE:
+ITEM_SERIALIZER = 'item-serializer.pck'
 
-# The first file contains enough information about each CVSRevision to
-# deduce preliminary Changesets.  The second file is a sorted version
-# of the first.
-CVS_REVS_SUMMARY_DATAFILE = 'revs-summary.txt'
-CVS_REVS_SUMMARY_SORTED_DATAFILE = 'revs-summary-s.txt'
+# The first file contains the CVSRevisions in a form that can be
+# sorted to deduce preliminary Changesets.  The second file is the
+# sorted version of the first.
+CVS_REVS_DATAFILE = 'revs.dat'
+CVS_REVS_SORTED_DATAFILE = 'revs-s.dat'
 
-# The first file contains enough information about each CVSSymbol to
-# deduce preliminary Changesets.  The second file is a sorted version
-# of the first.
-CVS_SYMBOLS_SUMMARY_DATAFILE = 'symbols-summary.txt'
-CVS_SYMBOLS_SUMMARY_SORTED_DATAFILE = 'symbols-summary-s.txt'
+# The first file contains the CVSSymbols in a form that can be sorted
+# to deduce preliminary Changesets.  The second file is the sorted
+# version of the first.
+CVS_SYMBOLS_DATAFILE = 'symbols.dat'
+CVS_SYMBOLS_SORTED_DATAFILE = 'symbols-s.dat'
 
 # A mapping from CVSItem id to Changeset id.
 CVS_ITEM_TO_CHANGESET = 'cvs-item-to-changeset.dat'
@@ -125,8 +124,8 @@ MIRROR_NODES_STORE = 'mirror-nodes.pck'
 # from symbol_id to file offset.
 SYMBOL_OFFSETS_DB = 'symbol-offsets.pck'
 
-# Pickled map of CVSFile.id to instance.
-CVS_FILES_DB = 'cvs-files.pck'
+# Pickled map of CVSPath.id to instance.
+CVS_PATHS_DB = 'cvs-paths.pck'
 
 # A series of records.  The first is a pickled serializer.  Each
 # subsequent record is a serialized list of all CVSItems applying to a
@@ -198,12 +197,6 @@ RCS_DELTAS_STORE = 'rcs-deltas.pck'
 RCS_TREES_INDEX_TABLE = 'rcs-trees-index.dat'
 RCS_TREES_STORE = 'rcs-trees.pck'
 
-# Records the revision tree of each RCS file after removing revisions
-# belonging to excluded branches.  Note that the branch ordering is arbitrary
-# in this file.
-RCS_TREES_FILTERED_INDEX_TABLE = 'rcs-trees-filtered-index.dat'
-RCS_TREES_FILTERED_STORE = 'rcs-trees-filtered.pck'
-
 # At any given time during OutputPass, holds the full text of each CVS
 # revision that was checked out already and still has descendants that will
 # be checked out.
@@ -211,11 +204,6 @@ CVS_CHECKOUT_DB = 'cvs-checkout.db'
 
 # End of DBs related to --use-internal-co.
 
-# If this run will output directly to a Subversion repository, then
-# this is the name of the file that each revision will temporarily be
-# written to prior to writing it into the repository.
-DUMPFILE = 'svn.dump'
-
 # flush a commit if a 5 minute gap occurs.
 COMMIT_THRESHOLD = 5 * 60
 
