@@ -1,7 +1,7 @@
 # (Be in -*- python -*- mode.)
 #
 # ====================================================================
-# Copyright (c) 2006-2007 CollabNet.  All rights reserved.
+# Copyright (c) 2006-2008 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -22,13 +22,15 @@ import re
 from cvs2svn_lib.common import InternalError
 from cvs2svn_lib.common import FatalError
 from cvs2svn_lib.context import Ctx
-from cvs2svn_lib.log import Log
+from cvs2svn_lib.log import logger
 from cvs2svn_lib.symbol import Trunk
 from cvs2svn_lib.symbol import Branch
 from cvs2svn_lib.symbol import Tag
 from cvs2svn_lib.symbol import ExcludedSymbol
 from cvs2svn_lib.cvs_item import CVSRevision
 from cvs2svn_lib.cvs_item import CVSRevisionModification
+from cvs2svn_lib.cvs_item import CVSRevisionAdd
+from cvs2svn_lib.cvs_item import CVSRevisionChange
 from cvs2svn_lib.cvs_item import CVSRevisionAbsent
 from cvs2svn_lib.cvs_item import CVSRevisionNoop
 from cvs2svn_lib.cvs_item import CVSSymbol
@@ -113,7 +115,7 @@ class LODItems(object):
 
 
 class CVSFileItems(object):
-  def __init__(self, cvs_file, trunk, cvs_items):
+  def __init__(self, cvs_file, trunk, cvs_items, original_ids=None):
     # The file whose data this instance holds.
     self.cvs_file = cvs_file
 
@@ -132,14 +134,26 @@ class CVSFileItems(object):
       if isinstance(cvs_item, CVSRevision) and cvs_item.prev_id is None:
         self.root_ids.add(cvs_item.id)
 
+    # self.original_ids is a dict {cvs_rev.rev : cvs_rev.id} holding
+    # the IDs originally allocated to each CVS revision number.  This
+    # member is stored for the convenience of RevisionCollectors.
+    if original_ids is not None:
+      self.original_ids = original_ids
+    else:
+      self.original_ids = {}
+      for cvs_item in cvs_items:
+        if isinstance(cvs_item, CVSRevision):
+          self.original_ids[cvs_item.rev] = cvs_item.id
+
   def __getstate__(self):
-    return (self.cvs_file.id, self.values(),)
+    return (self.cvs_file.id, self.values(), self.original_ids,)
 
   def __setstate__(self, state):
-    (cvs_file_id, cvs_items,) = state
-    cvs_file = Ctx()._cvs_file_db.get_file(cvs_file_id)
+    (cvs_file_id, cvs_items, original_ids,) = state
+    cvs_file = Ctx()._cvs_path_db.get_path(cvs_file_id)
     CVSFileItems.__init__(
         self, cvs_file, cvs_file.project.get_trunk(), cvs_items,
+        original_ids=original_ids,
         )
 
   def add(self, cvs_item):
@@ -167,7 +181,7 @@ class CVSFileItems(object):
       try:
         cvs_item.check_links(self)
       except AssertionError:
-        Log().error(
+        logger.error(
             'Link consistency error in %s\n'
             'This is probably a bug internal to cvs2svn.  Please file a bug\n'
             'report including the following stack trace (see FAQ for more '
@@ -241,16 +255,16 @@ class CVSFileItems(object):
 
     LOD is the LineOfDevelopment where the iteration should start.
     CVS_BRANCH is the CVSBranch instance that starts the LOD if any;
-    otherwise it is None.  ID is the id of the first CVSRevision on
-    this LOD, or None if there are none.
+    otherwise it is None.  START_ID is the id of the first CVSRevision
+    on this LOD, or None if there are none.
 
     There are two cases handled by this routine: trunk (where LOD is a
-    Trunk instance, CVS_BRANCH is None, and ID is the id of the 1.1
-    revision) and a branch (where LOD is a Branch instance, CVS_BRANCH
-    is a CVSBranch instance, and ID is either the id of the first
-    CVSRevision on the branch or None if there are no CVSRevisions on
-    the branch).  Note that CVS_BRANCH and ID cannot simultaneously be
-    None.
+    Trunk instance, CVS_BRANCH is None, and START_ID is the id of the
+    1.1 revision) and a branch (where LOD is a Branch instance,
+    CVS_BRANCH is a CVSBranch instance, and START_ID is either the id
+    of the first CVSRevision on the branch or None if there are no
+    CVSRevisions on the branch).  Note that CVS_BRANCH and START_ID cannot
+    simultaneously be None.
 
     Yield an LODItems instance for each line of development."""
 
@@ -330,19 +344,44 @@ class CVSFileItems(object):
       for lod_items in self._iter_tree(lod, cvs_branch, id):
         yield lod_items
 
+  def iter_deltatext_ancestors(self, cvs_rev):
+    """Generate the delta-dependency ancestors of CVS_REV.
+
+    Generate then ancestors of CVS_REV in deltatext order; i.e., back
+    along branches towards trunk, then outwards along trunk towards
+    HEAD."""
+
+    while True:
+      # Determine the next candidate source revision:
+      if isinstance(cvs_rev.lod, Trunk):
+        if cvs_rev.next_id is None:
+          # HEAD has no ancestors, so we are done:
+          return
+        else:
+          cvs_rev = self[cvs_rev.next_id]
+      else:
+        cvs_rev = self[cvs_rev.prev_id]
+
+      yield cvs_rev
+
   def _sever_branch(self, lod_items):
     """Sever the branch from its source and discard the CVSBranch.
 
     LOD_ITEMS describes a branch that should be severed from its
     source, deleting the CVSBranch and creating a new root.  Also set
-    LOD_ITEMS.cvs_branch to none.
+    LOD_ITEMS.cvs_branch to None.
+
+    If LOD_ITEMS has no source (e.g., because it is the trunk branch
+    or because it has already been severed), do nothing.
 
     This method can only be used before symbols have been grafted onto
     CVSBranches.  It does not adjust NTDBR, NTDBR_PREV_ID or
     NTDBR_NEXT_ID even if LOD_ITEMS describes a NTDB."""
 
     cvs_branch = lod_items.cvs_branch
-    assert cvs_branch is not None
+    if cvs_branch is None:
+      return
+
     assert not cvs_branch.tag_ids
     assert not cvs_branch.branch_ids
     source_rev = self[cvs_branch.source_id]
@@ -451,7 +490,7 @@ class CVSFileItems(object):
     if rev_1_2_id is not None:
       raise VendorBranchError(
           'File \'%s\' has default branch=%s but also a revision %s'
-          % (self.cvs_file.filename,
+          % (self.cvs_file.rcs_path,
              vendor_lod_items.cvs_branch.branch_number, self[rev_1_2_id].rev,)
           )
 
@@ -526,7 +565,7 @@ class CVSFileItems(object):
       cvs_branch = vendor_lod_items.cvs_branch
       rev_1_1 = self[cvs_branch.source_id]
       assert isinstance(rev_1_1, CVSRevision)
-      Log().debug('Removing unnecessary revision %s' % (rev_1_1,))
+      logger.debug('Removing unnecessary revision %s' % (rev_1_1,))
 
       # Delete the 1.1.1 CVSBranch and sever the vendor branch from trunk:
       self._sever_branch(vendor_lod_items)
@@ -556,25 +595,37 @@ class CVSFileItems(object):
         cvs_rev2 = self[id]
         cvs_rev2.prev_id = cvs_rev.id
 
-  def _delete_unneeded(self, cvs_item, metadata_db):
-    if isinstance(cvs_item, CVSRevisionNoop) \
-           and cvs_item.rev == '1.1' \
-           and isinstance(cvs_item.lod, Trunk) \
-           and len(cvs_item.branch_ids) >= 1 \
-           and self[cvs_item.branch_ids[0]].next_id is not None \
-           and not cvs_item.closed_symbols \
-           and not cvs_item.ntdbr:
-      # FIXME: This message will not match if the RCS file was renamed
-      # manually after it was created.
-      log_msg = metadata_db[cvs_item.metadata_id].log_msg
-      cvs_generated_msg = 'file %s was initially added on branch %s.\n' % (
-          self.cvs_file.basename,
-          self[cvs_item.branch_ids[0]].symbol.name,)
-      return log_msg == cvs_generated_msg
-    else:
+  def _is_unneeded_initial_trunk_delete(self, cvs_item, metadata_db):
+    if not isinstance(cvs_item, CVSRevisionNoop):
+      # This rule can only be applied to dead revisions.
       return False
 
-  def remove_unneeded_deletes(self, metadata_db):
+    if cvs_item.rev != '1.1':
+      return False
+
+    if not isinstance(cvs_item.lod, Trunk):
+      return False
+
+    if cvs_item.closed_symbols:
+      return False
+
+    if cvs_item.ntdbr:
+      return False
+
+    log_msg = metadata_db[cvs_item.metadata_id].log_msg
+    return bool(
+        re.match(
+            r'file .* was initially added on branch .*\.\n$',
+            log_msg,
+            )
+        or re.match(
+            # This variant commit message was reported by one user:
+            r'file .* was added on branch .*\n$',
+            log_msg,
+            )
+        )
+
+  def remove_unneeded_initial_trunk_delete(self, metadata_db):
     """Remove unneeded deletes for this file.
 
     If a file is added on a branch, then a trunk revision is added at
@@ -583,61 +634,54 @@ class CVSFileItems(object):
 
     for id in self.root_ids:
       cvs_item = self[id]
-      if self._delete_unneeded(cvs_item, metadata_db):
-        Log().debug('Removing unnecessary delete %s' % (cvs_item,))
+      if self._is_unneeded_initial_trunk_delete(cvs_item, metadata_db):
+        logger.debug('Removing unnecessary delete %s' % (cvs_item,))
 
-        # Delete cvs_item:
+        # Sever any CVSBranches rooted at cvs_item.
+        for cvs_branch_id in cvs_item.branch_ids[:]:
+          cvs_branch = self[cvs_branch_id]
+          self._sever_branch(self.get_lod_items(cvs_branch))
+
+        # Tagging a dead revision doesn't do anything, so remove any
+        # CVSTags that refer to cvs_item:
+        while cvs_item.tag_ids:
+          del self[cvs_item.tag_ids.pop()]
+
+        # Now delete cvs_item itself:
         self.root_ids.remove(cvs_item.id)
-        del self[id]
+        del self[cvs_item.id]
         if cvs_item.next_id is not None:
           cvs_rev_next = self[cvs_item.next_id]
           cvs_rev_next.prev_id = None
           self.root_ids.add(cvs_rev_next.id)
 
-        # Delete all CVSBranches rooted at this revision.  If there is
-        # a CVSRevision on the branch, it should already be an add so
-        # it doesn't have to be changed.
-        for cvs_branch_id in cvs_item.branch_ids:
-          cvs_branch = self[cvs_branch_id]
-          del self[cvs_branch.id]
+        # This can only happen once per file, so we're done:
+        return
 
-          if cvs_branch.next_id is not None:
-            cvs_branch_next = self[cvs_branch.next_id]
-            cvs_branch_next.first_on_branch_id = None
-            cvs_branch_next.prev_id = None
-            self.root_ids.add(cvs_branch_next.id)
-
-        # Tagging a dead revision doesn't do anything, so remove any
-        # tags that were set on 1.1:
-        for cvs_tag_id in cvs_item.tag_ids:
-          del self[cvs_tag_id]
-
-        # This can only happen once per file, and we might have just
-        # changed self.root_ids, so break out of the loop:
-        break
-
-  def _initial_branch_delete_unneeded(self, lod_items, metadata_db):
+  def _is_unneeded_initial_branch_delete(self, lod_items, metadata_db):
     """Return True iff the initial revision in LOD_ITEMS can be deleted."""
 
-    if lod_items.cvs_branch is not None \
-           and lod_items.cvs_branch.source_id is not None \
-           and len(lod_items.cvs_revisions) >= 2:
-      cvs_revision = lod_items.cvs_revisions[0]
-      cvs_rev_source = self[lod_items.cvs_branch.source_id]
-      if isinstance(cvs_revision, CVSRevisionAbsent) \
-             and not cvs_revision.tag_ids \
-             and not cvs_revision.branch_ids \
-             and abs(cvs_revision.timestamp - cvs_rev_source.timestamp) <= 2:
-        # FIXME: This message will not match if the RCS file was renamed
-        # manually after it was created.
-        log_msg = metadata_db[cvs_revision.metadata_id].log_msg
-        return bool(re.match(
-            r'file %s was added on branch .* on '
-            r'\d{4}\-\d{2}\-\d{2} \d{2}\:\d{2}\:\d{2}( [\+\-]\d{4})?'
-            '\n' % (re.escape(self.cvs_file.basename),),
-            log_msg,
-            ))
-    return False
+    if not lod_items.cvs_revisions:
+      return False
+
+    cvs_revision = lod_items.cvs_revisions[0]
+
+    if cvs_revision.ntdbr:
+      return False
+
+    if not isinstance(cvs_revision, CVSRevisionAbsent):
+      return False
+
+    if cvs_revision.branch_ids:
+      return False
+
+    log_msg = metadata_db[cvs_revision.metadata_id].log_msg
+    return bool(re.match(
+        r'file .* was added on branch .* on '
+        r'\d{4}\-\d{2}\-\d{2} \d{2}\:\d{2}\:\d{2}( [\+\-]\d{4})?'
+        '\n$',
+        log_msg,
+        ))
 
   def remove_initial_branch_deletes(self, metadata_db):
     """If the first revision on a branch is an unnecessary delete, remove it.
@@ -651,24 +695,30 @@ class CVSFileItems(object):
     revision."""
 
     for lod_items in self.iter_lods():
-      if self._initial_branch_delete_unneeded(lod_items, metadata_db):
+      if self._is_unneeded_initial_branch_delete(lod_items, metadata_db):
         cvs_revision = lod_items.cvs_revisions[0]
-        Log().debug(
+        logger.debug(
             'Removing unnecessary initial branch delete %s' % (cvs_revision,)
             )
-        cvs_branch = lod_items.cvs_branch
-        cvs_rev_source = self[cvs_branch.source_id]
-        cvs_rev_next = lod_items.cvs_revisions[1]
 
-        # Delete cvs_revision:
+        # Sever the branch from its source if necessary:
+        self._sever_branch(lod_items)
+
+        # Delete the first revision on the branch:
+        self.root_ids.remove(cvs_revision.id)
         del self[cvs_revision.id]
-        cvs_rev_next.prev_id = None
-        self.root_ids.add(cvs_rev_next.id)
-        cvs_rev_source.branch_commit_ids.remove(cvs_revision.id)
 
-        # Delete the CVSBranch on which it is located:
-        del self[cvs_branch.id]
-        cvs_rev_source.branch_ids.remove(cvs_branch.id)
+        # If it had a successor, adjust its backreference and add it
+        # to the root_ids:
+        if cvs_revision.next_id is not None:
+          cvs_rev_next = self[cvs_revision.next_id]
+          cvs_rev_next.prev_id = None
+          self.root_ids.add(cvs_rev_next.id)
+
+        # Tagging a dead revision doesn't do anything, so remove any
+        # tags that were set on it:
+        for tag_id in cvs_revision.tag_ids:
+          del self[tag_id]
 
   def _exclude_tag(self, cvs_tag):
     """Exclude the specified CVS_TAG."""
@@ -752,8 +802,7 @@ class CVSFileItems(object):
         rev_1_1 = self.get(first_rev.prev_id)
         rev_1_2 = self.get(last_rev.ntdbr_next_id)
 
-        if lod_items.cvs_branch is not None:
-          self._sever_branch(lod_items)
+        self._sever_branch(lod_items)
 
         if rev_1_1 is not None:
           rev_1_1.next_id = first_rev.id
@@ -808,11 +857,8 @@ class CVSFileItems(object):
     if ntdbr_excluded:
       self.graft_ntdbr_to_trunk()
 
-  def filter_excluded_symbols(self, revision_excluder):
-    """Delete any excluded symbols and references to them.
-
-    Call the revision_excluder's callback methods to let it know what
-    is being excluded."""
+  def filter_excluded_symbols(self):
+    """Delete any excluded symbols and references to them."""
 
     ntdbr_excluded = False
     for lod_items in self.iter_lods():
@@ -835,8 +881,6 @@ class CVSFileItems(object):
     if ntdbr_excluded:
       self.graft_ntdbr_to_trunk()
 
-    revision_excluder.process_file(self)
-
   def _mutate_branch_to_tag(self, cvs_branch):
     """Mutate the branch CVS_BRANCH into a tag."""
 
@@ -847,7 +891,7 @@ class CVSFileItems(object):
     cvs_tag = CVSTag(
         cvs_branch.id, cvs_branch.cvs_file, cvs_branch.symbol,
         cvs_branch.source_lod, cvs_branch.source_id,
-        cvs_branch.revision_recorder_token,
+        cvs_branch.revision_reader_token,
         )
     self.add(cvs_tag)
     cvs_revision = self[cvs_tag.source_id]
@@ -860,7 +904,7 @@ class CVSFileItems(object):
     cvs_branch = CVSBranch(
         cvs_tag.id, cvs_tag.cvs_file, cvs_tag.symbol,
         None, cvs_tag.source_lod, cvs_tag.source_id, None,
-        cvs_tag.revision_recorder_token,
+        cvs_tag.revision_reader_token,
         )
     self.add(cvs_branch)
     cvs_revision = self[cvs_branch.source_id]
@@ -924,7 +968,7 @@ class CVSFileItems(object):
     parent = self[branch_id]
     assert isinstance(parent, CVSBranch)
 
-    Log().debug('Grafting %s from %s (on %s) onto %s' % (
+    logger.debug('Grafting %s from %s (on %s) onto %s' % (
                 cvs_tag, source, source.lod, parent,))
     # Switch parent:
     source.tag_ids.remove(cvs_tag.id)
@@ -973,7 +1017,7 @@ class CVSFileItems(object):
     parent = possible_parent
     assert isinstance(parent, CVSBranch)
 
-    Log().debug('Grafting %s from %s (on %s) onto %s' % (
+    logger.debug('Grafting %s from %s (on %s) onto %s' % (
                 cvs_branch, source, source.lod, parent,))
     # Switch parent:
     source.branch_ids.remove(cvs_branch.id)
@@ -993,7 +1037,11 @@ class CVSFileItems(object):
       for cvs_tag in lod_items.cvs_tags:
         self._adjust_tag_parent(cvs_tag)
 
-      for cvs_branch in lod_items.cvs_branches:
+      # It is important to process branches in reverse order, so that
+      # a branch graft target (which necessarily occurs earlier in the
+      # list than the branch itself) is not moved before the branch
+      # itself.
+      for cvs_branch in reversed(lod_items.cvs_branches):
         self._adjust_branch_parents(cvs_branch)
 
   def _get_revision_source(self, cvs_symbol):
@@ -1025,6 +1073,29 @@ class CVSFileItems(object):
             isinstance(source, CVSRevisionModification)
             ]
 
+  def determine_revision_properties(self, revision_property_setters):
+    """Set the properties and properties_changed fields on CVSRevisions."""
+
+    for lod_items in self.iter_lods():
+      for cvs_rev in lod_items.cvs_revisions:
+        cvs_rev.properties = {}
+        for revision_property_setter in revision_property_setters:
+          revision_property_setter.set_properties(cvs_rev)
+
+    for lod_items in self.iter_lods():
+      for cvs_rev in lod_items.cvs_revisions:
+        if isinstance(cvs_rev, CVSRevisionAdd):
+          cvs_rev.properties_changed = True
+        elif isinstance(cvs_rev, CVSRevisionChange):
+          prev_properties = self[
+              cvs_rev.get_effective_prev_id()
+              ].get_properties()
+          properties = cvs_rev.get_properties()
+
+          cvs_rev.properties_changed = properties != prev_properties
+        else:
+          cvs_rev.properties_changed = False
+
   def record_opened_symbols(self):
     """Set CVSRevision.opened_symbols for the surviving revisions."""
 
