@@ -1,7 +1,7 @@
 # (Be in -*- python -*- mode.)
 #
 # ====================================================================
-# Copyright (c) 2000-2007 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2008 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -22,20 +22,14 @@ import time
 import threading
 
 
-class Log:
+class _Log:
   """A Simple logging facility.
 
   If self.log_level is DEBUG or higher, each line will be timestamped
   with the number of wall-clock seconds since the time when this
   module was first imported.
 
-  If self.use_timestamps is True, each line will be timestamped with a
-  human-readable clock time.
-
-  The public methods of this class are thread-safe.
-
-  This class is a Borg; see
-  http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/66531."""
+  The public methods of this class are thread-safe."""
 
   # These constants represent the log levels that this class supports.
   # The increase_verbosity() and decrease_verbosity() methods rely on
@@ -49,17 +43,8 @@ class Log:
 
   start_time = time.time()
 
-  __shared_state = {}
-
   def __init__(self):
-    self.__dict__ = self.__shared_state
-    if self.__dict__:
-      return
-
-    self.log_level = Log.NORMAL
-
-    # Set this to True if you want to see timestamps on each line output.
-    self.use_timestamps = False
+    self.log_level = _Log.NORMAL
 
     # The output file to use for errors:
     self._err = sys.stderr
@@ -73,21 +58,21 @@ class Log:
   def increase_verbosity(self):
     self.lock.acquire()
     try:
-      self.log_level = min(self.log_level + 1, Log.DEBUG)
+      self.log_level = min(self.log_level + 1, _Log.DEBUG)
     finally:
       self.lock.release()
 
   def decrease_verbosity(self):
     self.lock.acquire()
     try:
-      self.log_level = max(self.log_level - 1, Log.ERROR)
+      self.log_level = max(self.log_level - 1, _Log.ERROR)
     finally:
       self.lock.release()
 
   def is_on(self, level):
     """Return True iff messages at the specified LEVEL are currently on.
 
-    LEVEL should be one of the constants Log.WARN, Log.QUIET, etc."""
+    LEVEL should be one of the constants _Log.WARN, _Log.QUIET, etc."""
 
     return self.log_level >= level
 
@@ -96,12 +81,9 @@ class Log:
 
     retval = []
 
-    if self.log_level >= Log.DEBUG:
+    if self.log_level >= _Log.DEBUG:
       retval.append('%f: ' % (time.time() - self.start_time,))
 
-    if self.use_timestamps:
-      retval.append(time.strftime('[%Y-%m-%d %I:%M:%S %Z] - '))
-
     return ''.join(retval)
 
   def _write(self, out, *args):
@@ -138,37 +120,41 @@ class Log:
   def error(self, *args):
     """Log a message at the ERROR level."""
 
-    if self.is_on(Log.ERROR):
+    if self.is_on(_Log.ERROR):
       self._write(self._err, *args)
 
   def warn(self, *args):
     """Log a message at the WARN level."""
 
-    if self.is_on(Log.WARN):
+    if self.is_on(_Log.WARN):
       self._write(self._out, *args)
 
   def quiet(self, *args):
     """Log a message at the QUIET level."""
 
-    if self.is_on(Log.QUIET):
+    if self.is_on(_Log.QUIET):
       self._write(self._out, *args)
 
   def normal(self, *args):
     """Log a message at the NORMAL level."""
 
-    if self.is_on(Log.NORMAL):
+    if self.is_on(_Log.NORMAL):
       self._write(self._out, *args)
 
   def verbose(self, *args):
     """Log a message at the VERBOSE level."""
 
-    if self.is_on(Log.VERBOSE):
+    if self.is_on(_Log.VERBOSE):
       self._write(self._out, *args)
 
   def debug(self, *args):
     """Log a message at the DEBUG level."""
 
-    if self.is_on(Log.DEBUG):
+    if self.is_on(_Log.DEBUG):
       self._write(self._out, *args)
 
 
+# Create an instance that everybody can use:
+logger = _Log()
+
+
