@@ -1,7 +1,7 @@
 # (Be in -*- python -*- mode.)
 #
 # ====================================================================
-# Copyright (c) 2000-2007 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2009 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -23,7 +23,7 @@ import gc
 from cvs2svn_lib import config
 from cvs2svn_lib.common import FatalError
 from cvs2svn_lib.context import Ctx
-from cvs2svn_lib.log import Log
+from cvs2svn_lib.log import logger
 from cvs2svn_lib.stats_keeper import StatsKeeper
 from cvs2svn_lib.stats_keeper import read_stats_keeper
 from cvs2svn_lib.artifact_manager import artifact_manager
@@ -44,13 +44,13 @@ def check_for_garbage():
   gc.set_debug(gc.DEBUG_SAVEALL)
   gc_count = gc.collect()
   if gc_count:
-    if Log().is_on(Log.DEBUG):
-      Log().debug(
+    if logger.is_on(logger.DEBUG):
+      logger.debug(
           'INTERNAL: %d unreachable object(s) were garbage collected:'
           % (gc_count,)
           )
       for g in gc.garbage:
-        Log().debug('    %s' % (g,))
+        logger.debug('    %s' % (g,))
     del gc.garbage[:]
 
 
@@ -64,7 +64,7 @@ class Pass(object):
   def register_artifacts(self):
     """Register artifacts (created and needed) in artifact_manager."""
 
-    raise NotImplementedError
+    raise NotImplementedError()
 
   def _register_temp_file(self, basename):
     """Helper method; for brevity only."""
@@ -82,7 +82,7 @@ class Pass(object):
     RUN_OPTIONS is an instance of RunOptions.  STATS_KEEPER is an
     instance of StatsKeeper."""
 
-    raise NotImplementedError
+    raise NotImplementedError()
 
 
 class PassManager:
@@ -124,8 +124,8 @@ class PassManager:
       return pass_number
     except ValueError:
       # Is pass_name the name of one of the passes?
-      for i in range(len(self.passes)):
-        if self.passes[i].name == pass_name:
+      for (i, the_pass) in enumerate(self.passes):
+        if the_pass.name == pass_name:
           return i + 1
       raise InvalidPassError('Unknown pass name (%r).' % (pass_name,))
 
@@ -166,7 +166,7 @@ class PassManager:
     start_time = time.time()
     for i in range(index_start, index_end):
       the_pass = self.passes[i]
-      Log().quiet('----- pass %d (%s) -----' % (i + 1, the_pass.name,))
+      logger.quiet('----- pass %d (%s) -----' % (i + 1, the_pass.name,))
       artifact_manager.pass_started(the_pass)
 
       if i == 0:
@@ -183,7 +183,7 @@ class PassManager:
       stats_keeper.log_duration_for_pass(
           end_time - start_time, i + 1, the_pass.name
           )
-      Log().normal(stats_keeper.single_pass_timing(i + 1))
+      logger.normal(stats_keeper.single_pass_timing(i + 1))
       stats_keeper.archive(
           artifact_manager.get_temp_file(config.STATISTICS_FILE % (i + 1,))
           )
@@ -199,8 +199,8 @@ class PassManager:
     for the_pass in self.passes[index_end:]:
       artifact_manager.pass_deferred(the_pass)
 
-    Log().quiet(stats_keeper)
-    Log().normal(stats_keeper.timings())
+    logger.quiet(stats_keeper)
+    logger.normal(stats_keeper.timings())
 
     # Consistency check:
     artifact_manager.check_clean()
@@ -209,7 +209,7 @@ class PassManager:
     """Output (to sys.stdout) the indices and names of available passes."""
 
     print 'PASSES:'
-    for i in range(len(self.passes)):
-      print '%5d : %s' % (i + 1, self.passes[i].name,)
+    for (i, the_pass) in enumerate(self.passes):
+      print '%5d : %s' % (i + 1, the_pass.name,)
 
 
