@@ -1,7 +1,7 @@
 # (Be in -*- python -*- mode.)
 #
 # ====================================================================
-# Copyright (c) 2000-2007 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2008 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -24,91 +24,141 @@ import ConfigParser
 from cStringIO import StringIO
 
 from cvs2svn_lib.common import warning_prefix
-from cvs2svn_lib.log import Log
+from cvs2svn_lib.log import logger
 
 
-class SVNPropertySetter:
-  """Abstract class for objects that can set properties on a SVNCommitItem."""
-
-  def set_properties(self, s_item):
-    """Set any properties that can be determined for S_ITEM.
-
-    S_ITEM is an instance of SVNCommitItem.  This method should modify
-    S_ITEM.svn_props in place."""
-
-    raise NotImplementedError
+def _squash_case(s):
+  return s.lower()
 
 
-class CVSRevisionNumberSetter(SVNPropertySetter):
-  """Set the cvs2svn:cvs-rev property to the CVS revision number."""
-
-  propname = 'cvs2svn:cvs-rev'
-
-  def set_properties(self, s_item):
-    if self.propname in s_item.svn_props:
-      return
-
-    s_item.svn_props[self.propname] = s_item.cvs_rev.rev
-    s_item.svn_props_changed = True
+def _preserve_case(s):
+  return s
 
 
-class ExecutablePropertySetter(SVNPropertySetter):
-  """Set the svn:executable property based on cvs_rev.cvs_file.executable."""
-
-  propname = 'svn:executable'
-
-  def set_properties(self, s_item):
-    if self.propname in s_item.svn_props:
-      return
-
-    if s_item.cvs_rev.cvs_file.executable:
-      s_item.svn_props[self.propname] = '*'
+def cvs_file_is_binary(cvs_file):
+  return cvs_file.mode == 'b'
 
 
-class CVSBinaryFileEOLStyleSetter(SVNPropertySetter):
+class FilePropertySetter(object):
+  """Abstract class for objects that set properties on a CVSFile."""
+
+  def maybe_set_property(self, cvs_file, name, value):
+    """Set a property on CVS_FILE if it does not already have a value.
+
+    This method is here for the convenience of derived classes."""
+
+    if name not in cvs_file.properties:
+      cvs_file.properties[name] = value
+
+  def set_properties(self, cvs_file):
+    """Set any properties needed for CVS_FILE.
+
+    CVS_FILE is an instance of CVSFile.  This method should modify
+    CVS_FILE.properties in place."""
+
+    raise NotImplementedError()
+
+
+class ExecutablePropertySetter(FilePropertySetter):
+  """Set the svn:executable property based on cvs_file.executable."""
+
+  def set_properties(self, cvs_file):
+    if cvs_file.executable:
+      self.maybe_set_property(cvs_file, 'svn:executable', '*')
+
+
+class DescriptionPropertySetter(FilePropertySetter):
+  """Set the cvs:description property based on cvs_file.description."""
+
+  def __init__(self, propname='cvs:description'):
+    self.propname = propname
+
+  def set_properties(self, cvs_file):
+    if cvs_file.description:
+      self.maybe_set_property(cvs_file, self.propname, cvs_file.description)
+
+
+class CVSBinaryFileEOLStyleSetter(FilePropertySetter):
   """Set the eol-style to None for files with CVS mode '-kb'."""
 
-  propname = 'svn:eol-style'
-
-  def set_properties(self, s_item):
-    if self.propname in s_item.svn_props:
-      return
-
-    if s_item.cvs_rev.cvs_file.mode == 'b':
-      s_item.svn_props[self.propname] = None
+  def set_properties(self, cvs_file):
+    if cvs_file.mode == 'b':
+      self.maybe_set_property(cvs_file, 'svn:eol-style', None)
 
 
-class MimeMapper(SVNPropertySetter):
+class MimeMapper(FilePropertySetter):
   """A class that provides mappings from file names to MIME types."""
 
   propname = 'svn:mime-type'
 
-  def __init__(self, mime_types_file):
+  def __init__(
+          self, mime_types_file=None, mime_mappings=None,
+          ignore_case=False
+          ):
+    """Constructor.
+
+    Arguments:
+
+      mime_types_file -- a path to a MIME types file on disk.  Each
+          line of the file should contain the MIME type, then a
+          whitespace-separated list of file extensions; e.g., one line
+          might be 'text/plain txt c h cpp hpp'.  (See
+          http://en.wikipedia.org/wiki/Mime.types for information
+          about mime.types files):
+
+      mime_mappings -- a dictionary mapping a file extension to a MIME
+          type; e.g., {'txt': 'text/plain', 'cpp': 'text/plain'}.
+
+      ignore_case -- True iff case should be ignored in filename
+          extensions.  Setting this option to True can be useful if
+          your CVS repository was used on systems with
+          case-insensitive filenames, in which case you might have a
+          mix of uppercase and lowercase filenames."""
+
     self.mappings = { }
+    if ignore_case:
+      self.transform_case = _squash_case
+    else:
+      self.transform_case = _preserve_case
 
-    for line in file(mime_types_file):
-      if line.startswith("#"):
-        continue
+    if mime_types_file is None and mime_mappings is None:
+      logger.error('Should specify MIME types file or dict.\n')
 
-      # format of a line is something like
-      # text/plain c h cpp
-      extensions = line.split()
-      if len(extensions) < 2:
-        continue
-      type = extensions.pop(0)
-      for ext in extensions:
+    if mime_types_file is not None:
+      for line in file(mime_types_file):
+        if line.startswith("#"):
+          continue
+
+        # format of a line is something like
+        # text/plain c h cpp
+        extensions = line.split()
+        if len(extensions) < 2:
+          continue
+        type = extensions.pop(0)
+        for ext in extensions:
+          ext = self.transform_case(ext)
+          if ext in self.mappings and self.mappings[ext] != type:
+            logger.error(
+                "%s: ambiguous MIME mapping for *.%s (%s or %s)\n"
+                % (warning_prefix, ext, self.mappings[ext], type)
+                )
+          self.mappings[ext] = type
+
+    if mime_mappings is not None:
+      for ext, type in mime_mappings.iteritems():
+        ext = self.transform_case(ext)
         if ext in self.mappings and self.mappings[ext] != type:
-          Log().error(
+          logger.error(
               "%s: ambiguous MIME mapping for *.%s (%s or %s)\n"
               % (warning_prefix, ext, self.mappings[ext], type)
               )
         self.mappings[ext] = type
 
-  def set_properties(self, s_item):
-    if self.propname in s_item.svn_props:
+  def set_properties(self, cvs_file):
+    if self.propname in cvs_file.properties:
       return
 
-    basename, extension = os.path.splitext(s_item.cvs_rev.cvs_file.basename)
+    basename, extension = os.path.splitext(cvs_file.rcs_basename)
 
     # Extension includes the dot, so strip it (will leave extension
     # empty if filename ends with a dot, which is ok):
@@ -120,12 +170,14 @@ class MimeMapper(SVNPropertySetter):
     if not extension:
       extension = basename
 
+    extension = self.transform_case(extension)
+
     mime_type = self.mappings.get(extension, None)
     if mime_type is not None:
-      s_item.svn_props[self.propname] = mime_type
+      cvs_file.properties[self.propname] = mime_type
 
 
-class AutoPropsPropertySetter(SVNPropertySetter):
+class AutoPropsPropertySetter(FilePropertySetter):
   """Set arbitrary svn properties based on an auto-props configuration.
 
   This class supports case-sensitive or case-insensitive pattern
@@ -180,14 +232,14 @@ class AutoPropsPropertySetter(SVNPropertySetter):
   def __init__(self, configfilename, ignore_case=True):
     config = ConfigParser.ConfigParser()
     if ignore_case:
-      self.transform_case = self.squash_case
+      self.transform_case = _squash_case
     else:
-      config.optionxform = self.preserve_case
-      self.transform_case = self.preserve_case
+      config.optionxform = _preserve_case
+      self.transform_case = _preserve_case
 
     configtext = open(configfilename).read()
     if self.comment_re.search(configtext):
-      Log().warn(
+      logger.warn(
           '%s: Please be aware that a space followed by a\n'
           'semicolon is sometimes treated as a comment in configuration\n'
           'files.  This pattern was seen in\n'
@@ -210,16 +262,10 @@ class AutoPropsPropertySetter(SVNPropertySetter):
           if value:
             self._add_pattern(pattern, value)
 
-  def squash_case(self, s):
-    return s.lower()
-
-  def preserve_case(self, s):
-    return s
-
   def _add_pattern(self, pattern, props):
     propdict = {}
     if self.quoted_re.match(pattern):
-      Log().warn(
+      logger.warn(
           '%s: Quoting is not supported in auto-props; please verify rule\n'
           'for %r.  (Using pattern including quotation marks.)\n'
           % (warning_prefix, pattern,)
@@ -229,7 +275,7 @@ class AutoPropsPropertySetter(SVNPropertySetter):
       m = self.property_unset_re.match(prop)
       if m:
         name = m.group('name')
-        Log().debug(
+        logger.debug(
             'auto-props: For %r, leaving %r unset.' % (pattern, name,)
             )
         propdict[name] = None
@@ -240,13 +286,13 @@ class AutoPropsPropertySetter(SVNPropertySetter):
         name = m.group('name')
         value = m.group('value')
         if self.quoted_re.match(value):
-          Log().warn(
+          logger.warn(
               '%s: Quoting is not supported in auto-props; please verify\n'
               'rule %r for pattern %r.  (Using value\n'
               'including quotation marks.)\n'
               % (warning_prefix, prop, pattern,)
               )
-        Log().debug(
+        logger.debug(
             'auto-props: For %r, setting %r to %r.' % (pattern, name, value,)
             )
         propdict[name] = value
@@ -255,14 +301,14 @@ class AutoPropsPropertySetter(SVNPropertySetter):
       m = self.property_novalue_re.match(prop)
       if m:
         name = m.group('name')
-        Log().debug(
+        logger.debug(
             'auto-props: For %r, setting %r to the empty string'
             % (pattern, name,)
             )
         propdict[name] = ''
         continue
 
-      Log().warn(
+      logger.warn(
           '%s: in auto-props line for %r, value %r cannot be parsed (ignored)'
           % (warning_prefix, pattern, prop,)
           )
@@ -270,14 +316,14 @@ class AutoPropsPropertySetter(SVNPropertySetter):
     self.patterns.append(self.Pattern(self.transform_case(pattern), propdict))
 
   def get_propdict(self, cvs_file):
-    basename = self.transform_case(cvs_file.basename)
+    basename = self.transform_case(cvs_file.rcs_basename)
     propdict = {}
     for pattern in self.patterns:
       if pattern.match(basename):
         for (key,value) in pattern.propdict.items():
           if key in propdict:
             if propdict[key] != value:
-              Log().warn(
+              logger.warn(
                   "Contradictory values set for property '%s' for file %s."
                   % (key, cvs_file,))
           else:
@@ -285,34 +331,32 @@ class AutoPropsPropertySetter(SVNPropertySetter):
 
     return propdict
 
-  def set_properties(self, s_item):
-    propdict = self.get_propdict(s_item.cvs_rev.cvs_file)
+  def set_properties(self, cvs_file):
+    propdict = self.get_propdict(cvs_file)
     for (k,v) in propdict.items():
-      if k in s_item.svn_props:
-        if s_item.svn_props[k] != v:
-          Log().warn(
+      if k in cvs_file.properties:
+        if cvs_file.properties[k] != v:
+          logger.warn(
               "Property '%s' already set to %r for file %s; "
               "auto-props value (%r) ignored."
-              % (k, s_item.svn_props[k], s_item.cvs_rev.cvs_path, v,))
+              % (k, cvs_file.properties[k], cvs_file.cvs_path, v,)
+              )
       else:
-        s_item.svn_props[k] = v
+        cvs_file.properties[k] = v
 
 
-class CVSBinaryFileDefaultMimeTypeSetter(SVNPropertySetter):
+class CVSBinaryFileDefaultMimeTypeSetter(FilePropertySetter):
   """If the file is binary and its svn:mime-type property is not yet
   set, set it to 'application/octet-stream'."""
 
-  propname = 'svn:mime-type'
-
-  def set_properties(self, s_item):
-    if self.propname in s_item.svn_props:
-      return
-
-    if s_item.cvs_rev.cvs_file.mode == 'b':
-      s_item.svn_props[self.propname] = 'application/octet-stream'
+  def set_properties(self, cvs_file):
+    if cvs_file.mode == 'b':
+      self.maybe_set_property(
+          cvs_file, 'svn:mime-type', 'application/octet-stream'
+          )
 
 
-class EOLStyleFromMimeTypeSetter(SVNPropertySetter):
+class EOLStyleFromMimeTypeSetter(FilePropertySetter):
   """Set svn:eol-style based on svn:mime-type.
 
   If svn:mime-type is known but svn:eol-style is not, then set
@@ -322,64 +366,109 @@ class EOLStyleFromMimeTypeSetter(SVNPropertySetter):
 
   propname = 'svn:eol-style'
 
-  def set_properties(self, s_item):
-    if self.propname in s_item.svn_props:
+  def set_properties(self, cvs_file):
+    if self.propname in cvs_file.properties:
       return
 
-    if s_item.svn_props.get('svn:mime-type', None) is not None:
-      if s_item.svn_props['svn:mime-type'].startswith("text/"):
-        s_item.svn_props[self.propname] = 'native'
+    mime_type = cvs_file.properties.get('svn:mime-type', None)
+    if mime_type:
+      if mime_type.startswith("text/"):
+        cvs_file.properties[self.propname] = 'native'
       else:
-        s_item.svn_props[self.propname] = None
+        cvs_file.properties[self.propname] = None
 
 
-class DefaultEOLStyleSetter(SVNPropertySetter):
+class DefaultEOLStyleSetter(FilePropertySetter):
   """Set the eol-style if one has not already been set."""
 
-  propname = 'svn:eol-style'
+  valid_values = {
+      None : None,
+      # Also treat "binary" as None:
+      'binary' : None,
+      'native' : 'native',
+      'CRLF' : 'CRLF', 'LF' : 'LF', 'CR' : 'CR',
+      }
 
   def __init__(self, value):
     """Initialize with the specified default VALUE."""
 
-    self.value = value
+    try:
+      # Check that value is valid, and translate it to the proper case
+      self.value = self.valid_values[value]
+    except KeyError:
+      raise ValueError(
+          'Illegal value specified for the default EOL option: %r' % (value,)
+          )
 
-  def set_properties(self, s_item):
-    if self.propname in s_item.svn_props:
-      return
-
-    s_item.svn_props[self.propname] = self.value
+  def set_properties(self, cvs_file):
+    self.maybe_set_property(cvs_file, 'svn:eol-style', self.value)
 
 
-class SVNBinaryFileKeywordsPropertySetter(SVNPropertySetter):
+class SVNBinaryFileKeywordsPropertySetter(FilePropertySetter):
   """Turn off svn:keywords for files with binary svn:eol-style."""
 
   propname = 'svn:keywords'
 
-  def set_properties(self, s_item):
-    if self.propname in s_item.svn_props:
+  def set_properties(self, cvs_file):
+    if self.propname in cvs_file.properties:
       return
 
-    if not s_item.svn_props.get('svn:eol-style'):
-      s_item.svn_props[self.propname] = None
+    if not cvs_file.properties.get('svn:eol-style'):
+      cvs_file.properties[self.propname] = None
 
 
-class KeywordsPropertySetter(SVNPropertySetter):
+class KeywordsPropertySetter(FilePropertySetter):
   """If the svn:keywords property is not yet set, set it based on the
   file's mode.  See issue #2."""
 
-  propname = 'svn:keywords'
-
   def __init__(self, value):
     """Use VALUE for the value of the svn:keywords property if it is
     to be set."""
 
     self.value = value
 
-  def set_properties(self, s_item):
-    if self.propname in s_item.svn_props:
+  def set_properties(self, cvs_file):
+    if cvs_file.mode in [None, 'kv', 'kvl']:
+      self.maybe_set_property(cvs_file, 'svn:keywords', self.value)
+
+
+class ConditionalPropertySetter(object):
+  """Delegate to the passed property setters when the passed predicate applies.
+  The predicate should be a function that takes a CVSFile or CVSRevision
+  argument and return True if the property setters should be applied."""
+
+  def __init__(self, predicate, *property_setters):
+    self.predicate = predicate
+    self.property_setters = property_setters
+
+  def set_properties(self, cvs_file_or_rev):
+    if self.predicate(cvs_file_or_rev):
+      for property_setter in self.property_setters:
+        property_setter.set_properties(cvs_file_or_rev)
+
+
+class RevisionPropertySetter:
+  """Abstract class for objects that can set properties on a CVSRevision."""
+
+  def set_properties(self, cvs_rev):
+    """Set any properties that can be determined for CVS_REV.
+
+    CVS_REV is an instance of CVSRevision.  This method should modify
+    CVS_REV.properties in place."""
+
+    raise NotImplementedError()
+
+
+class CVSRevisionNumberSetter(RevisionPropertySetter):
+  """Store the CVS revision number to an SVN property."""
+
+  def __init__(self, propname='cvs2svn:cvs-rev'):
+    self.propname = propname
+
+  def set_properties(self, cvs_rev):
+    if self.propname in cvs_rev.properties:
       return
 
-    if s_item.cvs_rev.cvs_file.mode in [None, 'kv', 'kvl']:
-      s_item.svn_props[self.propname] = self.value
+    cvs_rev.properties[self.propname] = cvs_rev.rev
 
 
