@@ -1,7 +1,7 @@
 # (Be in -*- python -*- mode.)
 #
 # ====================================================================
-# Copyright (c) 2000-2007 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2008 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -19,12 +19,11 @@
 
 from cvs2svn_lib.common import FatalError
 from cvs2svn_lib.process import check_command_runs
-from cvs2svn_lib.process import PipeStream
 from cvs2svn_lib.process import CommandFailedException
-from cvs2svn_lib.revision_manager import RevisionReader
+from cvs2svn_lib.abstract_rcs_revision_manager import AbstractRCSRevisionReader
 
 
-class RCSRevisionReader(RevisionReader):
+class RCSRevisionReader(AbstractRCSRevisionReader):
   """A RevisionReader that reads the contents via RCS."""
 
   def __init__(self, co_executable):
@@ -36,16 +35,14 @@ class RCSRevisionReader(RevisionReader):
                        'Please check that co is installed and in your PATH\n'
                        '(it is a part of the RCS software).' % (e,))
 
-  def get_content_stream(self, cvs_rev, suppress_keyword_substitution=False):
-    pipe_cmd = [
+  def get_pipe_command(self, cvs_rev, k_option):
+    return [
         self.co_executable,
         '-q',
         '-x,v',
         '-p%s' % (cvs_rev.rev,)
+        ] + k_option + [
+        cvs_rev.cvs_file.rcs_path
         ]
-    if suppress_keyword_substitution:
-      pipe_cmd.append('-kk')
-    pipe_cmd.append(cvs_rev.cvs_file.filename)
-    return PipeStream(pipe_cmd)
 
 
