@@ -1,7 +1,7 @@
 # (Be in -*- python -*- mode.)
 #
 # ====================================================================
-# Copyright (c) 2000-2007 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2008 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -41,7 +41,7 @@ import mmap
 from cvs2svn_lib.common import DB_OPEN_READ
 from cvs2svn_lib.common import DB_OPEN_WRITE
 from cvs2svn_lib.common import DB_OPEN_NEW
-from cvs2svn_lib.log import Log
+from cvs2svn_lib.log import logger
 
 
 # A unique value that can be used to stand for "unset" without
@@ -258,7 +258,7 @@ class RecordTable(AbstractRecordTable):
     self._limit_written = self._limit
 
   def flush(self):
-    Log().debug('Flushing cache for %s' % (self,))
+    logger.debug('Flushing cache for %s' % (self,))
 
     pairs = [(i, s) for (i, (dirty, s)) in self._cache.items() if dirty]
 
