@@ -1,7 +1,7 @@
 # (Be in -*- python -*- mode.)
 #
 # ====================================================================
-# Copyright (c) 2000-2007 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2008 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -21,7 +21,7 @@ import cPickle
 from cvs2svn_lib import config
 from cvs2svn_lib.common import error_prefix
 from cvs2svn_lib.common import FatalException
-from cvs2svn_lib.log import Log
+from cvs2svn_lib.log import logger
 from cvs2svn_lib.artifact_manager import artifact_manager
 from cvs2svn_lib.symbol import Trunk
 from cvs2svn_lib.symbol import IncludedSymbol
@@ -141,6 +141,20 @@ class _Stats:
     # revision where the branch is rooted:
     register(parent_cvs_rev.lod)
 
+    # If the parent revision is a non-trunk default (vendor) branch
+    # revision, then count trunk as a possible parent.  In particular,
+    # the symbol could be grafted to the post-commit that copies the
+    # vendor branch changes to trunk.  On the other hand, our vendor
+    # branch handling is currently too stupid to do so.  On the other
+    # other hand, when the vendor branch is being excluded from the
+    # conversion, then the vendor branch revision will be moved to
+    # trunk, again making trunk a possible parent--and *this* our code
+    # can handle.  In the end, considering trunk a possible parent can
+    # never affect the correctness of the conversion, and on balance
+    # seems to improve the selection of symbol parents.
+    if parent_cvs_rev.ntdbr:
+      register(cvs_file_items.trunk)
+
     # Any other branches that are rooted at the same revision and
     # were committed earlier than the branch are also possible
     # parents:
@@ -167,6 +181,13 @@ class _Stats:
     # revision where the branch is rooted:
     register(parent_cvs_rev.lod)
 
+    # If the parent revision is a non-trunk default (vendor) branch
+    # revision, then count trunk as a possible parent.  See the
+    # comment by the analogous code in
+    # register_branch_possible_parents() for more details.
+    if parent_cvs_rev.ntdbr:
+      register(cvs_file_items.trunk)
+
     # Branches that are rooted at the same revision are also
     # possible parents:
     for branch_id in parent_cvs_rev.branch_ids:
@@ -329,7 +350,7 @@ class SymbolStatisticsCollector:
 
     for stats in self._stats.values():
       if stats.is_ghost():
-        Log().warn('Deleting ghost symbol: %s' % (stats.lod,))
+        logger.warn('Deleting ghost symbol: %s' % (stats.lod,))
         del self._stats[stats.lod]
 
   def close(self):
@@ -399,7 +420,7 @@ class SymbolStatistics:
   def _check_blocked_excludes(self, symbol_map):
     """Check for any excluded LODs that are blocked by non-excluded symbols.
 
-    If any are found, describe the problem to Log().error() and raise
+    If any are found, describe the problem to logger.error() and raise
     a FatalException."""
 
     # A list of (lod,[blocker,...]) tuples for excludes that are
@@ -428,7 +449,7 @@ class SymbolStatistics:
         for blocker in lod_blockers:
           s.append('    %s\n' % (blocker,))
       s.append('\n')
-      Log().error(''.join(s))
+      logger.error(''.join(s))
 
       raise FatalException()
 
@@ -441,7 +462,7 @@ class SymbolStatistics:
     any tags with commits are found, output error messages describing
     the problems then raise a FatalException."""
 
-    Log().quiet("Checking for forced tags with commits...")
+    logger.quiet("Checking for forced tags with commits...")
 
     invalid_tags = [ ]
     for symbol in symbol_map.itervalues():
@@ -463,7 +484,7 @@ class SymbolStatistics:
     for tag in invalid_tags:
       s.append('    %s\n' % (tag.name))
     s.append('\n')
-    Log().error(''.join(s))
+    logger.error(''.join(s))
 
     raise FatalException()
 
@@ -472,7 +493,7 @@ class SymbolStatistics:
 
     SYMBOL_MAP is a map {AbstractSymbol : (Trunk|TypedSymbol)}
     indicating how each AbstractSymbol is to be converted.  If any
-    problems are detected, describe the problem to Log().error() and
+    problems are detected, describe the problem to logger.error() and
     raise a FatalException."""
 
     # We want to do all of the consistency checks even if one of them
@@ -488,7 +509,7 @@ class SymbolStatistics:
         try:
           stats.check_preferred_parent_allowed(lod)
         except SymbolPlanException, e:
-          Log().error('%s\n' % (e,))
+          logger.error('%s\n' % (e,))
           error_found = True
 
     try:
